# Generated by Django 1.9.2 on 2016-02-05 17:27

import phonenumber_field.modelfields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('two_factor', '0003_auto_20150817_1733'),
    ]

    operations = [
        migrations.AlterField(
            model_name='phonedevice',
            name='number',
            field=phonenumber_field.modelfields.PhoneNumberField(max_length=128),
        ),
    ]
