{% extends "two_factor/_base.html" %}

{% block content_wrapper %}
  <div class="container">
    <div class="row">
      <div class="col-md-5 col-md-offset-3">
        {% block content %}{% endblock %}
      </div>
    </div>
  </div>
{% endblock %}
