{% extends "two_factor/_base_focus.html" %}
{% load i18n %}

{% block content %}
  <h1>{% block title %}{% trans "Disable Two-factor Authentication" %}{% endblock %}</h1>
  <p>{% blocktrans trimmed %}You are about to disable two-factor authentication. This
    weakens your account security, are you sure?{% endblocktrans %}</p>
  <form method="post">
    {% csrf_token %}
    <table>{{ form }}</table>
    <button class="btn btn-danger"
            type="submit">{% trans "Disable" %}</button>
  </form>
{% endblock %}
